#!/bin/bash
#
# Last update: 2024.12.12

clear
echo "===================================================================================================="
echo ""
echo "此為 Ubuntu 22.04 LTS 系統自動安裝 cPanel 程式 (Client Only)"
echo ""
echo "注意事項:"
echo "01.不支援安裝 CloudLinux 套件(包含 KernelCare)，因為 Ubuntu 不支援"
echo "02.PHP 不支援 lsapi，改使用 mod_fcgid"
echo "03.PHP 目前支援 73 74 80 81 82 83 84"
echo "04.MySQL 僅支援 8.0，MariaDB 僅支援 10.6 & 10.11"
echo ""
echo "===================================================================================================="
sleep 10


# 如果系統為 CentOS or Almalinux，會直接離開 Script
if [ -e /etc/redhat-release ]; then
   echo "此主機系統為 CentOS or Almalinux，無法使用此 Script"
   exit
fi

# 如果系統已經安裝cPanel，會直接離開 Script
if [ -e /usr/local/cpanel/cpanel ]; then
   echo "此主機已經安裝 cPanel，無法使用此 Script"
   exit
fi

# 紀錄開始安裝時間
echo "開始安裝 : `date`" > /root/install_date

###############################
### 安裝 cPanel 基礎環境系統 ###
###############################

# 將 "[root@cpXX ~]#" 改成 "root@cpXX [~]#" (只會設定一次)
# \u 使用者名稱 \h 主機名稱 \w 目前工作目錄
if [ ! -e /usr/local/src/set_ps1 ]; then
   PS1="\u@\h [\w]# "
   echo 'PS1="\u@\h [\w]# "' >> ~/.bashrc
   touch /usr/local/src/set_ps1
fi

# 設定 ls alias，讓 ll 指令可顯示隱藏檔案 (只會設定一次)
if [ ! -e /usr/local/src/set_ls_alias ]; then
   echo "LS_OPTIONS='--color=tty -F -a -b -T 0';" >> /etc/bashrc
   echo "export LS_OPTIONS;" >> /etc/bashrc
   alias ll='ls -l --color=tty -F -a -b -T 0'
   alias ls='/bin/ls $LS_OPTIONS'
   touch /usr/local/src/set_ls_alias
fi

# 設定時區區域為 Taipei (只會設定一次)
if [ ! -e /usr/local/src/set_timezone ]; then
   /usr/bin/timedatectl set-timezone Asia/Taipei
   touch /usr/local/src/set_timezone
fi

# 設定 nameserver (只會詢問一次)
if [ ! -e /usr/local/src/set_nameserver ]; then

ans=''
while [ "$ans"=="1" -o "$ans"=="2" -o "$ans"=="3" ]; do
   clear
   echo "==================================================="
   echo "  請選擇主機存放位置，以便設定正確的 nameserver    "
   echo "                                                   "
   echo "  (1) 遠傳 (2) 宏遠 (3) 國外主機                   "
   echo "==================================================="
   read -e -p "您的選擇是 ? ==> (1/2/3): " -i "" ans
   case $ans in
        1)
           sed -i '/nameserver/d' /etc/resolv.conf
           echo "nameserver 103.17.9.155" >> /etc/resolv.conf
           echo "nameserver 103.17.11.155" >> /etc/resolv.conf
           echo "nameserver 168.95.1.1" >> /etc/resolv.conf
           touch /usr/local/src/set_nameserver
           break;;
        2)
           sed -i '/nameserver/d' /etc/resolv.conf
           echo "nameserver 168.95.1.1" >> /etc/resolv.conf
           echo "nameserver 8.8.8.8" >> /etc/resolv.conf
           touch /usr/local/src/set_nameserver
           break;;
        3)
           sed -i '/nameserver/d' /etc/resolv.conf
           echo "nameserver 8.8.8.8" >> /etc/resolv.conf
           echo "nameserver 4.2.2.1" >> /etc/resolv.conf
           touch /usr/local/src/set_nameserver
           break;;
        *)
           echo ""
           echo "輸入選擇為不正確 : $ans，請參考以下說明，輸入正確值"
           echo ""
           echo "(1) 遠傳 (2) 宏遠 & 溪城 (3) 國外主機"
           sleep 3
           continue;;
   esac
done

fi

# 系統基本設定
if [ ! -e /usr/local/src/os_basic_config ]; then
   # 停用 needrestart 服務
   if [ -f /etc/needrestart/needrestart.conf ]; then
      echo "\$nrconf{kernelhints} = 0;" >> /etc/needrestart/conf.d/temp-disable-needrestart.conf
      echo "\$nrconf{restart} = 'l';" >> /etc/needrestart/conf.d/temp-disable-needrestart.conf
   fi

   # Change the operating state
   systemctl set-default multi-user.target

   # 更新系統套件並設定 Cron
   apt update ; apt -y upgrade
   /usr/bin/echo "0 6 * * * root apt update ; apt -y upgrade > /dev/null 2>&1" >> /etc/crontab

   # 關閉 SElinux
   /usr/bin/sed -i 's/^SELINUX=.*/SELINUX=disabled/g' /etc/selinux/config
   /usr/sbin/setenforce 0

   # 優化 sysctl.conf 設定
   echo '' >> /etc/sysctl.conf
   /usr/bin/cat /opt/cpinst/cpinst/config/sysctl_conf >> /etc/sysctl.conf
   /usr/sbin/sysctl -p

   # fix CSF MESSENGER Bug
   systemctl stop nftables
   systemctl disable nftables

   touch /usr/local/src/os_basic_config
fi

# 安裝與設定 ntpdate 時間對時 (只會設定一次)
apt install ntpdate -y
/usr/sbin/ntpdate tock.stdtime.gov.tw
echo "0 1 * * * root /usr/sbin/ntpdate tock.stdtime.gov.tw > /dev/null 2>&1" >> /etc/crontab

# 關閉 ufw 防火牆 (只會設定一次)
if [ ! -e /usr/local/src/disable_ufw ]; then
   iptables-save > ~/firewall.rules
   systemctl stop ufw.service
   systemctl disable ufw.service
   touch /usr/local/src/disable_ufw
fi

# 關閉 Core Dump
cat /etc/security/limits.conf  |grep -v  . -n -m 1 |cut -d : -f1 |xargs -i  sed -i  '{}c \\*\thard\tcore\t0' /etc/security/limits.conf
echo 'kernel.core_pattern = /dev/null/' >> /etc/sysctl.conf
echo 'fs.suid_dumpable = 0' >> /etc/sysctl.conf
sysctl -p
echo 'ulimit -S -c 0 > /dev/null 2>&1' >> /etc/profile

# 安裝必要套件 (只會安裝一次)
if [ ! -e /usr/local/src/install_need_package ]; then
   # 安裝必要套件
   apt install traceroute net-tools vim curl mutt wget -y

   touch /usr/local/src/install_need_package
fi

############################
### 顯示安裝 cPanel 選單 ###
############################

# 定義各項目預設參數

# 1.外網網卡名稱
CPANEL_INTERFACE=`ip -o link show | awk -F': ' '{print $2}'|grep -v lo|head -n 1`

# 2.Main IP
MAIN_IP=`ifconfig $CPANEL_INTERFACE | grep -Eo 'inet (addr:)?([0-9]*\.){3}[0-9]*' | grep -Eo '([0-9]*\.){3}[0-9]*' | grep -v '127.0.0.1'`

# 3.Main Share IP
MAIN_SHARE_IP="$MAIN_IP"

# 4.Hostname
SERVER_HOSTNAME=`hostname`

# 5.第一組 DNS 主機名稱
NS0=""

# 6.第二組 DNS 主機名稱
NS1=""

# 7.接收 System 通知郵件
CUSTOMER_EMAIL="請手動輸入正確資訊"

# 8.接收 cPanel 通知郵件
WHM_EMAIL="請手動輸入正確資訊"

# 9.PHP版本(套用的 EA4 Json 設定檔)
eapver_version="83"

# 10.安裝 CXS
installcxs="no"

# 11.安裝 Installatron
installinstallatron="no"

# 12.安裝 KernelCare
kernelupdaten="no"

# 13.安裝 Zabbix
zabbix="no"

# 14.每小時寄信限制
mail_qty="300"

# 15.資料庫類型
mysql_version="8.0"

# 依照選擇主機使用者類型，顯示不同確定清單

ans=''
while [ "$ans"=="y" -o "$ans"=="q" ]; do
   clear
   echo "========================================================================"
   echo "  以下為預設選項設定，請務必確認無誤，再進行下一步                      "
   echo "                                                                        "

   echo "1.外網網卡名稱 : ${CPANEL_INTERFACE}"

   echo "2.Main IP : ${MAIN_IP}"

   echo "3.Main Share IP : ${MAIN_SHARE_IP}"

   echo "4.Hostname : ${SERVER_HOSTNAME}"

   echo "5.第一組 DNS 主機名稱 : ${NS0}"

   echo "6.第二組 DNS 主機名稱 : ${NS1}"

   echo "7.接收 System 通知郵件(管理員信箱) : ${CUSTOMER_EMAIL}"

   echo "8.接收 cPanel 通知郵件(管理員信箱) : ${WHM_EMAIL}"

   echo "9.PHP版本 : ${eapver_version}"

   echo "10.安裝 CXS : ${installcxs}"

   echo "11.安裝 Installatron : ${installinstallatron}"

   echo "12.安裝 KernelCare : ${kernelupdaten}"

   echo "13.安裝 Zabbix : ${zabbix}"

   echo "14.每小時寄信限制 : ${mail_qty}"

   echo "15.資料庫類型 : ${mysql_version}"

   echo "                                                                        "
   echo "  輸入項目編號，即可修改選項設定                                        "
   echo "                                                                        "
   echo "  (y) 開始安裝 cPanel (n) 取消安裝 cPanel                               "
   echo "                                                                        "
   echo "========================================================================"
   read -e -p "您的選擇是 ?  " -i "" ans
   case $ans in
        1)
           clear
           echo ""
           echo "### 請改成正確的外網網卡名稱 ###"
           echo ""
           read -e -p "1.外網網卡名稱 : " -i "$CPANEL_INTERFACE" CPANEL_INTERFACE
           continue;;
        2)
           clear
           echo ""
           read -e -p "2.Main IP : " -i "$MAIN_IP" MAIN_IP
           continue;;
        3)
           clear
           echo ""
           echo "### 管理服務主機，請設定客戶指定的 Main Share IP ###"
           echo ""
           read -e -p "3.Main Share IP : " -i "" MAIN_SHARE_IP
           continue;;
        4)
           clear
           echo ""
           echo "### 管理服務主機，請依照管理服務安裝需求表設定 ###"
           echo ""
           read -e -p "4.Hostname : " -i "$SERVER_HOSTNAME" SERVER_HOSTNAME
           continue;;
        5)
           clear
           echo ""
           echo "### 客戶有指定才需要設定 nameserver，否則留空即可 ###"
           echo ""
           read -e -p "5.第一組 DNS 主機名稱 : " -i "" NS0
           continue;;
        6)
           clear
           echo ""
           echo "### 客戶有指定才需要設定 nameserver，否則留空即可 ###"
           echo ""
           read -e -p "6.第二組 DNS 主機名稱 : " -i "" NS1
           continue;;
        7)
           clear
           echo ""
           echo "### 請設定 Linux 需求表 管理員信箱 ###"
           echo ""
           read -e -p "7.接收 System 通知郵件(管理員信箱) : " -i "$CUSTOMER_EMAIL" CUSTOMER_EMAIL
           continue;;
        8)
           clear
           echo ""
           echo "### 請設定 Linux 需求表 管理員信箱 ###"
           echo ""
           read -e -p "8.接收 cPanel 通知郵件(管理員信箱) : " -i "$WHM_EMAIL" WHM_EMAIL
           continue;;
        9)
           clear
           # https://support.cpanel.net/hc/en-us/articles/4405080880023-What-versions-of-PHP-are-available-in-EasyApache-4-on-Ubuntu
           echo ""
           echo "### 請依照管理服務安裝需求表設定 ###"
           echo "### Ubuntu 22.04 只支援 8.1 ~ 8.4 ###"
           echo ""
           echo "Apache 2.4 / PHP 8.1 / cgi = 81"
           echo "Apache 2.4 / PHP 8.2 / cgi = 82"
           echo "Apache 2.4 / PHP 8.3 / cgi = 83"
	        echo "Apache 2.4 / PHP 8.4 / cgi = 84"
           echo ""
           echo "例如 : PHP 8.3，就是輸入 83"
           echo ""
           read -e -p "9.PHP版本 : " -i "$eapver_version" eapver_version
           continue;;
        10)
           clear
           echo ""
           echo "安裝 = yes、不安裝 = no"
           echo ""
           read -e -p "10.安裝 CXS : " -i "$installcxs" installcxs
           continue;;
        11)
           clear
           echo ""
           echo "安裝 = yes、不安裝 = no"
           echo ""
           read -e -p "11.安裝 Installatron : " -i "$installinstallatron" installinstallatron
           continue;;
        12)
           clear
           echo ""
           echo "安裝 = yes、不安裝 = no"
           echo ""
           echo "### 單次部屬不安裝，Standard 管理服務要安裝 ###"
           echo ""
           read -e -p "12.安裝 KernelCare : " -i "$kernelupdaten" kernelupdaten
           continue;;
        13)
           clear
           echo ""
           echo "安裝 = yes、不安裝 = no"
           echo ""
           echo "### 單次部屬不安裝，Standard 管理服務要安裝 ###"
           echo ""
           read -e -p "13.安裝 Zabbix : " -i "$zabbix" zabbix
           continue;;
        14)
           clear
           echo ""
           echo "### 管理服務主機，請依照管理服務安裝需求表設定 ###"
           echo ""
           read -e -p "14.每小時寄信限制 : " -i "$mail_qty" mail_qty
           continue;;
        15)
           clear
           # https://support.cpanel.net/hc/en-us/articles/1500002121102-What-versions-of-MySQL-and-MariaDB-does-cPanel-support
           echo ""
           echo "### 目前 Ubuntu 22.04 支援 MySQL 8.0、MariaDB ###"
           echo "### MySQL 請輸入 8.0 ###"
           echo "### MariaDB 請輸入 10.6 or 10.11 or 11.4 ###"
           echo ""
           read -e -p "15.資料庫類型 : " -i "$mysql_version" mysql_version
           continue;;
        y)
           echo ""
           #echo "開始安裝 cPanel"

### 判斷輸入值是否正確，不正確不給安裝 ###

if [ "${MAIN_SHARE_IP}" = "請手動輸入正確資訊" ]; then
echo ""
echo "3.Main Share IP : 請手動輸入正確資訊"
sleep 3
continue
fi

if [ "${CUSTOMER_EMAIL}" = "請手動輸入正確資訊" ]; then
echo ""
echo "7.接收 cPanel 日誌郵件的電子郵件 : 請手動輸入正確資訊"
sleep 3
continue
fi

if [ "${WHM_EMAIL}" = "請手動輸入正確資訊" ]; then
echo ""
echo "8.提供聯絡信息的電子郵件 : 請手動輸入正確資訊"
sleep 3
continue
fi

if [ "${eapver_version}" != '81' ] && [ "${eapver_version}" != '82' ] && [ "${eapver_version}" != '83' ] && [ "${eapver_version}" != '84' ]; then
echo ""
echo "9.目前輸入 PHP 版本為 ${eapver_version} 無法使用，請重新輸入正確值"
sleep 3
continue
fi

if [ "${mysql_version}" != "8.0" ] && [ "${mysql_version}" != "10.6" ] && [ "${mysql_version}" != "10.11" ] && [ "${mysql_version}" != "11.4" ]; then
echo ""
echo "14.目前輸入 mysql 版本 ${mysql_version} 無法使用，請重新輸入正確值"
sleep 3
continue
fi

           break;;
        n)
           echo ""
           echo "取消安裝 cPanel !!!"
           echo ""
           exit;;
        *)
           echo ""
           echo "輸入選擇為不正確 : $ans，請參考以下說明，輸入正確值"
           echo ""
           echo "(y) 開始安裝 cPanel (n) 取消安裝 cPanel"
           echo ""
           echo "或者 輸入項目編號，即可修改選項設定"
           sleep 3
           continue;;
   esac
done

# 紀錄選項設定值
echo " " >> /root/install_date
echo "######################" >> /root/install_date
echo " " >> /root/install_date
echo "Use: ubuntu2204.sh" >> /root/install_date
echo " " >> /root/install_date
echo "1.外網網卡名稱 : ${CPANEL_INTERFACE}" >> /root/install_date
echo "2.Main IP : ${MAIN_IP}" >> /root/install_date
echo "3.Main Share IP : ${MAIN_SHARE_IP}" >> /root/install_date
echo "4.Hostname : ${SERVER_HOSTNAME}" >> /root/install_date
echo "5.第一組 DNS 主機名稱 : ${NS0}" >> /root/install_date
echo "6.第二組 DNS 主機名稱 : ${NS1}" >> /root/install_date
echo "7.接收 cPanel 日誌郵件的電子郵件 : ${CUSTOMER_EMAIL}" >> /root/install_date
echo "8.提供聯絡信息的電子郵件 : ${WHM_EMAIL}" >> /root/install_date
echo "9.PHP版本 : ${eapver_version}" >> /root/install_date
echo "10.安裝 CXS : ${installcxs}" >> /root/install_date
echo "11.安裝 Installatron : ${installinstallatron}" >> /root/install_date
echo "12.安裝 KernelCare : ${kernelupdaten}" >> /root/install_date
echo "13.安裝 Zabbix : ${zabbix}" >> /root/install_date
echo "14.每小時寄信限制 : ${mail_qty}" >> /root/install_date
echo "15.資料庫類型 : ${mysql_version}" >> /root/install_date
echo "######################" >> /root/install_date
echo " " >> /root/install_date

#######################
### 開始安裝 cPanel ###
#######################

# 設定正確 hostname
/usr/bin/hostnamectl set-hostname $SERVER_HOSTNAME

# 設定 cPanel 使用更新為 stable
echo "CPANEL=stable" > /etc/cpupdate.conf

# 複製 cpanel.config 範例設定檔
mkdir -p /root/cpanel_profile
cp /opt/cpinst/cpinst/config/cpanel.config /root/cpanel_profile/

# 客戶主機重啟Apache服務間隔改成預設值10秒
sed -i "s/^min_time_between_apache_graceful_restarts=.*/min_time_between_apache_graceful_restarts=10/g" /root/cpanel_profile/cpanel.config

# 設定每小時寄信最大限制數量 (客戶主機 才會進行設定，自用主機都是使用預設值 300)
sed -i "s/^maxemailsperhour=.*/maxemailsperhour=${mail_qty}/g" /root/cpanel_profile/cpanel.config

# 設定資料庫類型
sed -i "s/^mysql-version=.*/mysql-version=${mysql_version}/g" /root/cpanel_profile/cpanel.config

# 複製 EA4 Json 範例設定檔
eapver="cpanel_install_ea4_profile_php_${eapver_version}_ubuntu.json"
cp /opt/cpinst/cpinst/config/${eapver} /etc/cpanel_initial_install_ea4_profile.json

# 複製 wwwacct.conf 範例設定檔，置換正確的設定
cp /opt/cpinst/cpinst/config/wwwacct.conf /etc/wwwacct.conf
sed -i "s/QQHSTNME/${SERVER_HOSTNAME}/g" /etc/wwwacct.conf
sed -i "s/QQINTERFACE/${CPANEL_INTERFACE}/g" /etc/wwwacct.conf
sed -i "s/QQWHMMAIL/${WHM_EMAIL}/g" /etc/wwwacct.conf
sed -i "s/QQMAINSHAREIP/${MAIN_SHARE_IP}/g" /etc/wwwacct.conf
sed -i "s/QQNAM1/${NS0}/g" /etc/wwwacct.conf
sed -i "s/QQNAM2/${NS1}/g" /etc/wwwacct.conf

# 開始安裝 cPanel，安裝紀錄存放 /var/log/cpanel-install.log
cd /root && curl -o latest -L https://securedownloads.cpanel.net/latest && sh latest

# 更新 cPanel license & install snmpd
/usr/local/cpanel/cpkeyclt

####################
### WHM 初始設定 ###
####################

# 安裝 CSF 套件
apt remove ufw -y
apt install perl zip unzip libwww-perl sendmail-bin -y
/bin/bash /opt/cpinst/cpinst/install_script/csf.sh 2
systemctl start csf
systemctl start lfd
systemctl enable csf
systemctl enable lfd

# Apache mod_userdir Tweak
# 改成不吃 WHM 介面設定，透過 post 設定檔強制開啟全部使用者，包含 nobody
/usr/bin/cat <<EOF >> /usr/local/apache/conf/includes/post_virtualhost_global.conf
<IfModule mod_userdir.c>
  UserDir enabled
</IfModule>
EOF
/scripts/rebuildhttpdconf
/scripts/restartsrv_httpd

# 關閉 Compiler Access
/scripts/compilers off

# 關閉 cPHulk Brute Force Protection
cphulk=$(whmapi1 configureservice service='cphulkd' enabled='0' monitored='0')

# 設定 Edit System Mail Preferences
echo "cpanel: ${CUSTOMER_EMAIL}" > /etc/localaliases
echo 'nobody: /dev/null' >> /etc/localaliases
echo "root: ${CUSTOMER_EMAIL}" >> /etc/localaliases

# 複製 Contact Manager 範例設定檔
mv /var/cpanel/icontact_event_importance.json /var/cpanel/icontact_event_importance.json_bak
/bin/cp -f /opt/cpinst/cpinst/config/icontact_event_importance.json /var/cpanel/icontact_event_importance.json
/bin/cp -f /opt/cpinst/cpinst/config/clevels.conf /var/cpanel/clevels.conf

# Service Manager
servicemanager=$(whmapi1 configureservice service='cpdavd' enabled='0' monitored='0')

# Apache Configuration >> Global Configuration & DirectoryIndex Priority
sed -i 's/"root_options" : "[^"]*"/"root_options" : "FollowSymLinks IncludesNOEXEC ExecCGI"/' /etc/cpanel/ea4/ea4.conf
sed -i 's/"maxclients" : 150/"maxclients" : 256/' /etc/cpanel/ea4/ea4.conf
sed -i 's/"serverlimit" : 256/"serverlimit" : 256/' /etc/cpanel/ea4/ea4.conf
sed -i 's/directoryindex" : "index.php index.php8 index.php7 index.php5 index.perl index.pl index.plx index.ppl index.cgi index.jsp index.jp index.phtml index.shtml index.xhtml index.html index.htm index.js"/directoryindex" : "index.php index.php8 index.php7 index.php5 index.perl index.pl index.plx index.ppl index.cgi index.jsp index.jp index.phtml index.shtml index.xhtml index.html index.htm index.js Default.htm"/' /etc/cpanel/ea4/ea4.conf
/scripts/rebuildhttpdconf
/scripts/restartsrv_httpd

# Apache Configuration > Include Editor
apt install ea-apache24-mod_reqtimeout -y
/usr/bin/cat <<EOF >> /usr/local/apache/conf/includes/pre_main_global.conf
<IfModule mod_reqtimeout.c>
  RequestReadTimeout header=10-20,MinRate=500 body=20-600,MinRate=500
</IfModule>
EOF
/usr/bin/cat <<EOF >> /etc/apache2/conf.d/includes/pre_virtualhost_global.conf
  SSLUseStapling off
EOF
/scripts/rebuildhttpdconf
/scripts/restartsrv_httpd

# 安裝 RemoteIP 套件 (不確定項目 !!!)
/bin/bash /opt/cpinst/cpinst/install_script/remoteip.sh
sed -i '/add_remoteip/d' /etc/crontab
cp /opt/cpinst/cpinst/install_script/add_remoteip.sh /root/add_remoteip.sh
/bin/bash /root/add_remoteip.sh
echo "30 0 * * * root /bin/bash /root/add_remoteip.sh > /dev/null 2>&1" >> /etc/crontab

# 阻擋客戶網站被存取.git目錄與.git與.env檔案
cp /opt/cpinst/cpinst/config/deny_git.conf /etc/apache2/conf.d/
cp /opt/cpinst/cpinst/config/deny_env.conf /etc/apache2/conf.d/
/scripts/restartsrv_httpd

# Exim Configuration Manager
mkdir -p /var/cpanel/config.backups/
/bin/cp /opt/cpinst/cpinst/config/whm-config-backup-cpanel__smtp__exim-116.001000-1734056477.tar.gz /var/cpanel/config.backups/
/usr/local/cpanel/bin/cpconftool --restore=/var/cpanel/config.backups/whm-config-backup-cpanel__smtp__exim-116.001000-1734056477.tar.gz --modules=cpanel::smtp::exim
#echo '*: '`cat /var/cpanel/mainip` > /etc/mailips
#echo '*: '`hostname` > /etc/mailhelo

# MultiPHP INI Editor > ea-phpxx php.ini
# https://api.docs.cpanel.net/openapi/whm/operation/php_ini_set_directives/
multiphp=$(whmapi1 php_ini_set_directives directive-1='display_errors:Off' directive-2='disable_functions:symlink,show_source, system, exec, shell_exec, passthru, popen, proc_open, proc_close, proc_get_status, proc_nice, proc_terminate, escapeshellarg, escapeshellcmd, ini_alter, define_syslog_variables, posix_uname, posix_getpwuid, posix_getpwnam, apache_child_terminate, posix_kill, posix_mkfifo, posix_setpgid, posix_setsid, posix_setuid, proc_terminate, posix_uname, diskfreespace, leak, base64_encodem, eval, apache_child_terminate, apache_setenv' directive-3='date.timezone:Asia/Taipei' version="ea-php${eapver_version}")

# 關閉 PHP-FPM
phpfpm=$(whmapi1 php_set_default_accounts_to_fpm default_accounts_to_fpm='0')

# Background Process Killer
processtokill=$(whmapi1 configurebackgroundprocesskiller processes_to_kill='BitchX' processes_to_kill-0='bnc' processes_to_kill-1='eggdrop' processes_to_kill-2='generic-sniffers' processes_to_kill-3='guardservices' processes_to_kill-4='ircd' processes_to_kill-5='psyBNC' processes_to_kill-6='ptlink' processes_to_kill-7='services')

# 更新 /etc/my.cnf 設定檔
mv /etc/my.cnf /etc/my.cnf_bak
if [ "$mysql_version" == "8.0" ]; then
   cp /opt/cpinst/cpinst/config/my_8.txt /etc/my.cnf
   sed -i "s/^tmpdir/#tmpdir/g" /etc/my.cnf
   sed -i "s/^log-error/#log-error/g" /etc/my.cnf
fi
if [ "${mysql_version}" == "10.6" ] && [ "${mysql_version}" == "10.11" ] && [ "${mysql_version}" == "11.4" ]; then
   cp /opt/cpinst/cpinst/config/my.txt /etc/my.cnf
   sed -i '/optimizer_switch/d' /etc/my.cnf
fi
/scripts/restartsrv_mysql
ln -s /var/log/mysql/error.log /var/lib/mysql/`hostname`.err
ln -s /var/log/mysql/error.log /var/lib/mysql/error.err

# pure-ftpd
/scripts/setupftpserver --force pure-ftpd
rm -f /var/cpanel/conf/pureftpd/main.cache
mv /var/cpanel/conf/pureftpd/main /var/cpanel/conf/pureftpd/main.backup
/bin/cp -f /opt/cpinst/cpinst/config/pureftpd_main /var/cpanel/conf/pureftpd/main
touch /var/cpanel/conf/pureftpd/local
echo 'ForcePassiveIP: ~' >> /var/cpanel/conf/pureftpd/local
echo 'PassivePortRange: 30000 50000' >> /var/cpanel/conf/pureftpd/local
echo 'TLSCipherSuite: ECDHE-ECDSA-AES128-GCM-SHA256:ECDHE-RSA-AES128-GCM-SHA256:ECDHE-ECDSA-AES256-GCM-SHA384:ECDHE-RSA-AES256-GCM-SHA384:ECDHE-ECDSA-CHACHA20-POLY1305:ECDHE-RSA-CHACHA20-POLY1305:DHE-RSA-AES128-GCM-SHA256:DHE-RSA-AES256-GCM-SHA384:TLS_AES_256_GCM_SHA384:TLS_CHACHA20_POLY1305_SHA256:TLS_AES_128_GCM_SHA256' >> /var/cpanel/conf/pureftpd/local
sed -i "s/PassivePortRange 49152 65534/PassivePortRange 30000 50000/g" /etc/pure-ftpd.conf
/scripts/setupftpserver --force pure-ftpd
/scripts/restartsrv_ftpserver

# AutoSSL
autossl=$(whmapi1 --output=jsonpretty set_autossl_metadata  metadata_json='{"clobber_externally_signed":"0","notify_autossl_expiry":"0","notify_autossl_expiry_coverage":"0","notify_autossl_expiry_coverage_user":"0","notify_autossl_expiry_user":"0","notify_autossl_renewal":"0","notify_autossl_renewal_coverage":"0","notify_autossl_renewal_coverage_reduced":"0","notify_autossl_renewal_coverage_reduced_user":"0","notify_autossl_renewal_coverage_user":"0","notify_autossl_renewal_uncovered_domains":"0","notify_autossl_renewal_uncovered_domains_user":"0","notify_autossl_renewal_user":"0"}')

# Greylisting
greylisting=$(whmapi1 disable_cpgreylist)

# 啟用 Two-Factor Authentication
twofactor=$(whmapi1 twofactorauth_enable_policy)

# 設定 Default Style
whmapi1 set_default type='default' name='basic'
mkdir -pv /root/cpanel3-skel/.cpanel/nvdata/
echo -n "1" > /root/cpanel3-skel/.cpanel/nvdata/xmainNewStyleBannerDismissed

################
### 套件安裝 ###
################

# 安裝 Installatron
if [ "$installinstallatron" == "yes" ]; then
   # https://installatron.com/plugin/download/cpanelwhm
   cd ~
   wget https://data.installatron.com/installatron-plugin.sh
   chmod +x installatron-plugin.sh
   ./installatron-plugin.sh -f
fi

# 安裝 CXS
if [ "$installcxs" == "yes" ]; then
   # https://configserver.com/cp/cxs.html
   # https://configserver.com/cp/cxsinstaller.html
   /bin/bash /opt/cpinst/cpinst/install_script/cxs.sh
fi

# 安裝 KernelCare
if [ "$kernelupdaten" == "yes" ]; then
   curl -s https://repo.cloudlinux.com/kernelcare/kernelcare_install.sh | bash
   kcarectl --update
fi

# 安裝 SNMP
apt install snmpd snmp -y
mv -f /etc/snmp/snmpd.conf /etc/snmp/snmpd.conf.bak
wget -O /etc/snmp/snmpd.conf https://cpanel.net.tw/tool/snmp/snmpd-conf.txt
echo "syslocation "`uname -a` >>  /etc/snmp/snmpd.conf
systemctl restart snmpd
systemctl enable snmpd

# 安裝 Memcachd
apt install memcached -y
systemctl daemon-reload
systemctl enable memcached.service
systemctl start memcached.service
apt install ea-php${eapver_version}-php-memcache -y
apt install ea-php${eapver_version}-php-memcached -y

# 安裝 Zabbix
if [ "$zabbix" == "yes" ]; then
   cd /root
   git clone git@git.twnoc.net:nicole/zabbix.git
   /bin/bash /root/zabbix/zabbix_install.sh $SERVER_HOSTNAME
fi

# 安裝 mod_fcgid，設定主機全部 PHP 版本 Handlers 改成 fcgid (因為沒有lsapi，所以統一改用fcgi)
/opt/cpinst/cpinst/install_script/fcgi_ubuntu.sh

# Apache Configuration > Include Editor
/usr/bin/cat <<EOF >> /usr/local/apache/conf/includes/post_virtualhost_global.conf
<IfModule mod_fcgid.c>
# scan interval for busy timeout process
FcgidBusyScanInterval 90

# a FastCGI application will be killed after handling a request for FcgidBusyTimeout
FcgidBusyTimeout 600

# scan interval for exited process
FcgidErrorScanInterval 3

# Mirror the PHP cgi.fix_pathinfo setting
FcgidFixPathinfo 1

# scan interval for idle timeout process
FcgidIdleScanInterval 70

# An idle FastCGI application will be killed after FcgidIdleTimeout
FcgidIdleTimeout 360

# Communication timeout to FastCGI server
FcgidIOTimeout 1000

# maximum number of FastCGI application processes
FcgidMaxProcesses 1000

# Max process count of one class of FastCGI application
FcgidMaxProcessesPerClass 100

# maximum size of a request which will be held in memory
FcgidMaxRequestInMem 268435456

# maximum HTTP request length (128M)
FcgidMaxRequestLen 134217728

# Max requests handled by each FastCGI application
FcgidMaxRequestsPerProcess 0

# Min process count of one class of FastCGI application
FcgidMinProcessesPerClass 3

# CGI output buffer size
FcgidOutputBufferSize 1048576

# Header name which will be passed to FastCGI as environment variable
FcgidPassHeader HTTP_AUTHORIZATION

# maximum FastCGI application process lifetime
FcgidProcessLifeTime 300

# Each spawn adds this value to the process activity score
FcgidSpawnScore 1

# Maximum value of the process activity score which allows a spawn to occur
FcgidSpawnScoreUpLimit 10

# Each terminated process adds this value to the process activity score.
FcgidTerminationScore 2

# Amount subtracted from process activity score each second
FcgidTimeScore 2

# scan interval for zombie process
FcgidZombieScanInterval 3
</IfModule>
EOF
/scripts/rebuildhttpdconf
/scripts/restartsrv_httpd

################
### Fix Bug ###
################

# fix not open webmail for roundcube problem
/usr/local/cpanel/bin/update-roundcube-db

# 修正v106語系檔案消失問題 (v118沒有此問題，先暫時註解)
#/opt/cpinst/jacky_script/import_cplocales.sh

# remove WordPress-Toolkit-Plugin
# https://support.cpanel.net/hc/en-us/articles/360058369013-How-can-I-remove-the-WordPress-Toolkit-Plugin-
apt remove wp-toolkit-cpanel -y

# 再次設定正確 hostname，避免DNS未指向，被cPanel改成xxxx.cprapid.com名稱
/usr/bin/hostnamectl set-hostname $SERVER_HOSTNAME
/usr/local/cpanel/bin/set_hostname $SERVER_HOSTNAME
echo '*: '`hostname` > /etc/mailhelo
ln -s /var/log/mysql/error.log /var/lib/mysql/`hostname`.err

# 修正ipv6優先權高於ipv4問題
touch /etc/gai.conf
cat << EOF >> /etc/gai.conf
label ::1/128 0
label ::/0 1
label 2002::/16 2
label ::/96 3
label ::ffff:0:0/96 4
precedence ::1/128 50
precedence ::/0 40
precedence 2002::/16 30
precedence ::/96 20
precedence ::ffff:0:0/96 100
EOF

# 避免 whmapi1 指令執行失效，如果前面執行輸出結果為空，表示沒正常執行，故再執行一次
if [ -z "$cphulk" ]; then
   sleep 3
   whmapi1 configureservice service='cphulkd' enabled='0' monitored='0'
fi
if [ -z "$servicemanager" ]; then
   sleep 3
   whmapi1 configureservice service='cpdavd' enabled='0' monitored='0'
fi
if [ -z "$multiphp" ]; then
   sleep 3
   whmapi1 php_ini_set_directives directive-1='display_errors:Off' directive-2='disable_functions:symlink,show_source, system, exec, shell_exec, passthru, popen, proc_open, proc_close, proc_get_status, proc_nice, proc_terminate, escapeshellarg, escapeshellcmd, ini_alter, define_syslog_variables, posix_uname, posix_getpwuid, posix_getpwnam, apache_child_terminate, posix_kill, posix_mkfifo, posix_setpgid, posix_setsid, posix_setuid, proc_terminate, posix_uname, diskfreespace, leak, base64_encodem, eval, apache_child_terminate, apache_setenv' directive-3='date.timezone:Asia/Taipei' version="ea-php${eapver_version}"
fi
if [ -z "$phpfpm" ]; then
   sleep 3
   whmapi1 php_set_default_accounts_to_fpm default_accounts_to_fpm='0'
fi
if [ -z "$processtokill" ]; then
   sleep 3
   whmapi1 configurebackgroundprocesskiller processes_to_kill='BitchX' processes_to_kill-0='bnc' processes_to_kill-1='eggdrop' processes_to_kill-2='generic-sniffers' processes_to_kill-3='guardservices' processes_to_kill-4='ircd' processes_to_kill-5='psyBNC' processes_to_kill-6='ptlink' processes_to_kill-7='services'
fi
if [ -z "$autossl" ]; then
   sleep 3
   whmapi1 --output=jsonpretty set_autossl_metadata  metadata_json='{"clobber_externally_signed":"0","notify_autossl_expiry":"0","notify_autossl_expiry_coverage":"0","notify_autossl_expiry_coverage_user":"0","notify_autossl_expiry_user":"0","notify_autossl_renewal":"0","notify_autossl_renewal_coverage":"0","notify_autossl_renewal_coverage_reduced":"0","notify_autossl_renewal_coverage_reduced_user":"0","notify_autossl_renewal_coverage_user":"0","notify_autossl_renewal_uncovered_domains":"0","notify_autossl_renewal_uncovered_domains_user":"0","notify_autossl_renewal_user":"0"}'
fi
if [ -z "$greylisting" ]; then
   sleep 3
   whmapi1 disable_cpgreylist
fi
if [ -z "$twofactor" ]; then
   sleep 3
   whmapi1 twofactorauth_enable_policy
fi

###################
### Install End ###
###################

# 紀錄結束安裝時間
echo "結束安裝 : `date`" >> /root/install_date

# 說明已安裝完畢
echo ""
echo "ubuntu2204.sh 已執行安裝完畢，接著請依照 https://wiki.noc.tw/自用虛擬主機 初始設定進行手動設定與確認"
echo ""
echo "Zabbix 網頁頁面設定監控項目請參考 https://wiki.noc.tw/Zabbix_建立新主機"
echo ""

# Easyapache4 未正確套用指定PHP版本json檔案判斷

if [ ! -d "/etc/cpanel/ea4/profiles/custom/" ]; then

   echo "#########################"
   echo ""
   echo "注意 !!! 此次 cPanel 安裝 Easyapache4 並未正確套用指定 PHP版本 josn 檔案，請手動執行以下指令"
   echo ""
   echo "/usr/local/bin/ea_install_profile --install /etc/cpanel_initial_install_ea4_profile.json"
   echo ""
   echo "執行完畢後，進入 WHM > Easyapache4 確認目前預設 PHP版本 是否為安裝指定的版本"
   echo ""
   echo "接著執行 /opt/cpinst/cpinst/install_script/fcgi_ubuntu.sh 指令重新套用lsapi設定"
   echo ""
   echo "最後確認 WHM > MultiPHP Manager 功能底下 PHP Handlers 全部版本是否都改成 fcgid"
   echo ""
   echo "如果遇到不可預期的問題，請回報讓二線與主管知道這件事情，後續才能排查問題點 !!!"
   echo ""
   echo "#########################"

fi

# 程式設計人員將不定時確認cpinst部屬log，加以改善除錯，增加 cpinst 自動安裝穩定性
echo "cpinst 部屬完成寄送通知給程式設計人員，確認部屬過程中是否有錯誤，加以修正" | mutt -s "${MAIN_IP} 部屬 ${host_use_user} cPanel 主機環境 " -a /root/cpinst_install.log -a /root/install_date -- jacky@host.com.tw
